#include <acknex.h>
#include <default.c>

#define PRAGMA_PATH "sprites"

#define SIZE 4

BMAP *bmpFlake = "dirtflake.png";

typedef struct
{
	ENTITY *ent;
	int image;
} Tile;

Tile tiles[SIZE][SIZE];

int __id;
char *images[8];

ENTITY *selectedPair[2];

#define IS_SELECTED skill1
#define TILE_TYPE skill2
#define IS_DONE skill3
#define PROGRESS skill4

action tile()
{
	int id = __id;
	ENTITY *animal = ent_create(images[id], vector(my.x, my.y, my.z - 4), NULL);
	vec_fill(animal.scale_x, 0.125);
	set(animal, DECAL | LIGHT | UNLIT);
	animal.pan = 180;
	animal.tilt = 90;
	animal.flags2 |= UNTOUCHABLE;
	
	ENTITY *pile = ent_create("pile.mdl", my.x, NULL);
	pile.flags2 |= UNTOUCHABLE;
	
	my.TILE_TYPE = id;
	
	wait(1);
	
	my.PROGRESS = 0;
	
	while(1)
	{
		my.PROGRESS = clamp(my.PROGRESS + 20 * (2 * (my.IS_SELECTED || key_p) - 1) * time_step, 0, 100);
		ent_animate(pile, "dig", my.PROGRESS, 0);
		if(mouse_ent == me && my.IS_DONE == 0)
			pile.ambient = 50;
		else
			pile.ambient = 0;
		wait(1);
	}
}

function click_handler()
{
	if(mouse_ent == NULL)
		return;
	if(mouse_ent.IS_DONE)
		return;
	mouse_ent.IS_SELECTED = 1;
	if(selectedPair[0] == NULL)
	{
		selectedPair[0] = mouse_ent;
	}
	else
	{
		selectedPair[1] = mouse_ent;
		
		if(selectedPair[0].skill2 == selectedPair[1].skill2)
		{
			selectedPair[0].IS_DONE = 1;
			selectedPair[1].IS_DONE = 1;
		}
		
		on_mouse_left = NULL;
		while(selectedPair[0].PROGRESS < 100 || selectedPair[1].PROGRESS < 100)
		{
			wait(1);
		}
		wait(-0.3);
		on_mouse_left = click_handler;
		
		
		if(selectedPair[0].skill2 == selectedPair[1].skill2)
		{			
			selectedPair[0] = NULL;
			selectedPair[1] = NULL;	
			return;
		}
		selectedPair[0].IS_SELECTED = 0;
		selectedPair[1].IS_SELECTED = 0;
		
		
		selectedPair[0] = NULL;
		selectedPair[1] = NULL;
	}
}

function start_game()
{
	int i, x, y;
	level_load("outdoor.wmb");	

	int items[16];
	for(i = 0; i < 8; i++) {
		items[2*i + 0] = i;
		items[2*i + 1] = i;
	}
	for(i = 0; i < 100; i++) {
		int j,k;
		j = integer(random(16));
		k = integer(random(16));
		int t = items[j];
		items[j] = items[k];
		items[k] = t;
	}
	
	for(x = 0; x < SIZE; x++)
	{
		for(y = 0; y < SIZE; y++)
		{
			__id = items[SIZE * y + x];
			tiles[x][y].image = __id;
			tiles[x][y].ent = ent_create("tile.mdl", vector(64 * x, 64 * y, 0), tile);	
		}
	}
	
	camera.x = -32;
	camera.y = 98;
	camera.z = 204;
	camera.tilt = -65;
	camera.ambient = 20;
	camera.arc = 90;
	
	sun_angle.pan = 260;
}

function main()
{
	fps_max = 60;
	mouse_mode = 4;
	video_set(1280, 720, 32, 2);
	
	on_mouse_left = click_handler;
	
	images[0] = "chicken-0.png";
	images[1] = "cow-0.png";
	images[2] = "deer-0.png";
	images[3] = "duck-0.png";
	images[4] = "goat-0.png";
	images[5] = "goose-0.png";
	images[6] = "horse-0.png";
	images[7] = "sheep-0.png";
	
	start_game();
	
	on_r = start_game;
	
	
	while(1)
	{
		var isDone = 1;
		
		int x, y;
		for(x = 0; x < SIZE; x++)
		{
			for(y = 0; y < SIZE; y++)
			{
				isDone &= tiles[x][y].ent.IS_DONE;
			}
		}
		
		DEBUG_VAR(isDone, 16);
		
		wait(1);	
	}
}